import java.awt.Color;
import java.awt.event.KeyEvent;

/**
 * Ein einfaches Pacman-Spiel.
 * Der Pacman muss in einer leeren Welt vor den vier Gespenstern flchten.
 * 
 * @author Bernd Gramlich 
 * @version 2014-12-05
 */

public class Spiel
{
	private Vielfrass pacman;
	private Geist blinky, pinky, inky, clyde;
	private Beschriftung ueberschrift;
	
	/**
	 * Konstruktor.
	 * Erzeuge die vier Gespenster und den Pacman und bringe sie in Ausgangsposition.
	 */
	public Spiel ()
	{
	    Leinwand.erzeugeLeinwand ("Pacman 10c", 900, 700, Color.white);
		blinky = new Geist ("rot", "blau", 1, 1);
		pinky  = new Geist ("pink", "blau", 1, 1);
		inky   = new Geist ("tuerkis", "blau", 1, 1);
		clyde  = new Geist ("orange", "blau", 1, 1);
		pacman = new Vielfrass ();
		
		ueberschrift = new Beschriftung ();
		
		erzeugeAusgangsposition ();
	}

	/**
	 * Erzeuge die Ausgangsposition des Pacmanspiels.
	 * Der Pacman sitzt in der Mitte der Welt, die vier Gespenster an den Ecken.
	 */
	public void erzeugeAusgangsposition ()
	{
		blinky.bewegeNach (1, 1);
		pinky.bewegeNach  (9, 1);
		inky.bewegeNach   (9, 7);
		clyde.bewegeNach  (1, 7);
		pacman.bewegeNach (5, 4);
		
		blinky.werdeSichtbar ();
		pinky.werdeSichtbar ();
		inky.werdeSichtbar ();
		clyde.werdeSichtbar ();
		pacman.werdeSichtbar ();
		
		ueberschrift.setzeInhalt ("  Pacman  ");
		ueberschrift.setzeFarbe ("grau");
		ueberschrift.setzeSchriftgroesse (80);
		ueberschrift.setzePosition (300, 300);
		ueberschrift.macheSichtbar ();		
	}
	
	/**
	 * Spiele.
	 * Der Pacman horcht auf Tastaturbefehle mit den Pfeiltasten.
	 * Die Gespenster verfolgen ihn mit ihrer (noch unausgereiften) Methode jage ().
	 * Spielende ist momentan nach Drcken der Eingabetaste, weil die Gespenster den
	 * Pacman noch nicht fressen knnen.
	 */
	public void spiele ()
	{
	    /*
	     * Hilfsvariablen:
	     * spielfeld:  Die Leinwand, auf der wir spielen.
	     * aufhoeren:  Soll das Spiel jetzt beendet werden?
	     * bewegen:    Sollen die Figuren jetzt bewegt werden?
	     * taste:      Die letzte Taste, die auf der Leinwand gedrckt wurde (als
	     *             Tastaturereignis)
	     */
	    Leinwand spielfeld = Leinwand.gibLeinwand ();
        boolean aufhoeren = false;
        boolean bewegen;
        KeyEvent taste;
        
        /*
         * Lsche alle Tasten, die vor Spielbeginn gedrckt wurden.
         */
        while ((taste = spielfeld.gibTastendruck ()) != null);
        
        /*
         * Bringe die Figuren in Ausgangsposition, warte eine Sekunde
         * und mache dann die berschrift unsichtbar.
         */
        erzeugeAusgangsposition ();
        spielfeld.warte (1000);
        ueberschrift.macheUnsichtbar ();
        
        /*
         * Wiederhole, solange wir nicht aufhren mssen.
         */
        while (!aufhoeren)
        {
            /*
             * Solange keine Taste gedrckt wird, tue nichts.
             */
            while ((taste = spielfeld.gibTastendruck ()) == null);
            
            /*
             * Reagiere unterschiedlich, je nachdem, welche Taste gedrckt wurde.
             */
            switch (taste.getKeyCode())
            {
                /*
                 * Wenn eine der Tasten Pfeil nach oben, Pfeil nach links, Pfeil nach
                 * rechts oder Pfeil nach unten gedrckt wurde, dann wollen wir die
                 * Figuren bewegen. Wir drehen den Pacman in die passende
                 * Himmelsrichtung.
                 */
                case KeyEvent.VK_UP:    bewegen = true; pacman.blickeNachNorden (); break;
                case KeyEvent.VK_LEFT:  bewegen = true; pacman.blickeNachWesten (); break;
                case KeyEvent.VK_DOWN:  bewegen = true; pacman.blickeNachSueden (); break;
                case KeyEvent.VK_RIGHT: bewegen = true; pacman.blickeNachOsten  (); break;
                /*
                 * Wenn die Eingabetaste gedrckt wurde, wollen wir die Figuren nicht mehr
                 * bewegen, sondern mit dem Spielen aufhren.
                 */
                case KeyEvent.VK_ENTER: bewegen = false; aufhoeren = true; break;
                /*
                 * Wenn irgendeine andere Taste gedrckt wurde, bewegen wir die Figuren
                 * nicht.
                 */
                default:                bewegen = false;
            }
            
            /*
             * Wenn die Figuren bewegt werden sollen ...
             */
            if (bewegen)
            {
                /*
                 * ... dann macht zuerst der Pacman einen Schritt in seine Blickrichtung.
                 * Anschlieend versuchen alle vier Gespenster, ihn zu fangen.
                 */
                pacman.geheInnerhalb ();
                blinky.jage ();
                pinky.jage ();
                inky.jage ();
                clyde.jage ();
            }
        }
        
        /* 
         * Zu Spielende geben wir eine Nachricht aus, warten 3 Sekunden und machen dann
         * alle Figuren und die Nachricht unsichtbar.
         */
        ueberschrift.setzeInhalt ("Game over!");
		ueberschrift.macheSichtbar ();
		spielfeld.warte (3000);
		blinky.werdeUnsichtbar ();
		pinky.werdeUnsichtbar ();
		inky.werdeUnsichtbar ();
		clyde.werdeUnsichtbar ();
		pacman.werdeUnsichtbar ();
		ueberschrift.macheUnsichtbar ();
	}
}